****************************************************************************
****************************************************************************
**                                                                        **
**   Phoenix for TI-89, TI-92, and TI-92 Plus : Player Firing Routines    **
**                                                                        **
**   Copyright 1997-2000 by Patrick Davidson.  All Rights Reserved.       **
**                                                                        **
**   This file was last updated April 16, 2000                            **
**                                                                        **
**   For complete copyright information and contact addresses, refer to   **
**   the main source file "PHOENIX.ASM"                                   **
**                                                                        **
****************************************************************************
****************************************************************************

******************************************** SELECT PLAYER WEAPON
*
* Reads the function/number keys, and sets the selected weapon to the key
* pressed if that weapon is available.  Below are tables for both
* calculators.  The TI-89 table contains the masks as well as the bit numbers
* for each function key.  The TI-92 table contains only the masks becasue the
* bit is always 4 on it.
*
********

fkeys_table:
      IFND ti89
         dc.w     %1111111111111101
         dc.w     %1111111111110111
         dc.w     %1111111111011111
         dc.w     %1111111101111111
         dc.w     %1111110111111111
         dc.w     %1111111111111011
         dc.w     %1111111111101111
         dc.w     %1111111110111111
      ENDIF
      IFD ti89
         dc.w     %1111111111110111,3
         dc.w     %1111111111101111,3
         dc.w     %1111111111111011,2
         dc.w     %1111111111110111,2
         dc.w     %1111111111101111,2
         dc.w     %1111111111111011,1
         dc.w     %1111111111110111,1
         dc.w     %1111111111101111,1
      ENDIF

Player_Weapon_Select:
         lea      fkeys_table(pc),a0
         moveq    #7,d0
loop_pws:
      IFND ti89
         move.w   (a0)+,$18(a6)
         nop
         nop
         nop
         btst     #4,$1b(a6)
      ENDIF
      IFD ti89
         move.w   (a0)+,$18(a6)
         bsr      wait_keyport
         move.w   (a0)+,d1
         btst     d1,$1b(a6)
      ENDIF

         beq.s    select_weapon
         dbra     d0,loop_pws
         rts

select_weapon:
         btst     d0,weapons_available(a5)
         beq.s    not_available
         add.w    d0,d0
         move.w   d0,weapon_selected(a5)
not_available:
         rts

********************************************** FIRE PLAYER'S BULLETS
*
* If the a bullet should be fired (e.g. the player presses the fire button,
* or holds it long enough for a repeat), calls the routine from the table
* below to insert bullets into the bullet data structure.
*                                                   
********

Player_Bullets_Shoot:
      IFND ti89
         btst     #3,$1b(a6)
      ENDIF
      IFD ti89
         btst     #4,$1b(a6)
      ENDIF
         beq.s    player_fired
         clr.w    lock_status(a5)
         rts
already_fired:
         addq.w   #1,lock_status(a5)
         rts
player_fired:
         tst.w    lock_status(a5)
         bne.s    already_fired
         move.w   fire_delay(a5),lock_status(a5)
         move.w   weapon_selected(a5),d0
         move.w   player_xc(a5),d7
         move.w   player_yc(a5),d6
         add.w    player_weaponx(a5),d7
         move.w   Player_Weapon_Jump_Table(pc,d0.w),d0
         jmp      Player_Weapon_Jump_Table(pc,d0.w)

Player_Weapon_Jump_Table:
         dc.w     normal_weapon-Player_Weapon_Jump_Table
         dc.w     Double_Shoot-Player_Weapon_Jump_Table
         dc.w     Triple_Shoot-Player_Weapon_Jump_Table
         dc.w     Quadruple_Shoot-Player_Weapon_Jump_Table
         dc.w     Dual_Plasma_Cannon-Player_Weapon_Jump_Table
         dc.w     Golden_Arches-Player_Weapon_Jump_Table
         dc.w     Triple_Plasma_Cannon-Player_Weapon_Jump_Table
         dc.w     Ultimate_Weapon-Player_Weapon_Jump_Table

********************************************* NORMAL WEAPON HANDLERS

Ultimate_Weapon:
         moveq    #6,d5
         subq.w   #4,d6
         subq.w   #3,d7
         lea      Bullet_Ultimate2(pc),a1
         bsr      Fire_Bullet

         addq.w   #4,d7
         lea      Bullet_Ultimate3(pc),a1
         bsr      Fire_Bullet

         subq.w   #8,d7
         lea      Bullet_Ultimate1(pc),a1
         bsr      Fire_Bullet

         add.w    #12,d7
         lea      Bullet_Ultimate4(pc),a1
         bra      Fire_Bullet

Dual_Plasma_Cannon:
         moveq    #0,d5
         subq.w   #4,d6
         subq.w   #6,d7
         bsr.s    place_plasma_bullet
         addq.w   #8,d7
place_plasma_bullet:
         lea      Bullet_Plasma(pc),a1
         bra      Fire_Bullet

Triple_Plasma_Cannon:
         subq.w   #6,d7
         subq.w   #4,d6
         moveq    #-1,d5
         bsr.s    place_plasma_bullet
         addq.w   #4,d7
         moveq    #0,d5
         bsr.s    place_plasma_bullet
         addq.w   #4,d7
         moveq    #1,d5
         bra.s    place_plasma_bullet

Golden_Arches:
         addq.w   #6,d7
         addq.w   #4,d6
         lea      Bullet_Arch_Right(pc),a1
         moveq    #6,d5
         bsr      Fire_Bullet
         sub.w    #10,d7
         lea      Bullet_Arch_Left(pc),a1
         bra      Fire_Bullet

Quadruple_Shoot:
         bchg     #0,quad_fail(a5)
         subq.w   #7,d7
         bsr      QS_TwoShots
         add.w    #12,d7
QS_TwoShots:
         cmp.w    #2,player_ship(a5)
         bne.s    ok420
         move.w   d7,d4
         lsr.w    #2,d4
         add.b    quad_fail(a5),d4
         btst     #0,d4
         beq.s    rts420
ok420:   moveq    #10,d4
         moveq    #0,d5
         lea      Bullet_Quad_Left(pc),a1
         bsr      Fire_Bullet
         lea      Bullet_Quad_Right(pc),a1
         bra      Fire_Bullet

Triple_Shoot:
         subq.w   #1,d7
         subq.w   #5,d6
         lea      Bullet_Triple_Left(pc),a1
         bsr      Fire_Bullet
         lea      Bullet_Triple_Middle(pc),a1
         bsr      Fire_Bullet
         lea      Bullet_Triple_Right(pc),a1
         bra      Fire_Bullet

Double_Shoot:
         subq.w   #6,d6
         subq.w   #6,d7
         bsr.s    place_standard_bullet
         add.w    #10,d7
         bra.s    place_standard_bullet

normal_weapon:
         subq.w   #1,d7
         subq.w   #5,d6
place_standard_bullet:
         lea      Bullet_Standard(pc),a1

********************************************* FIRE A BULLET
*
* Insert bullet described at (A1) at coordinates (D7, D6) with data = D5
* Modifies D0, A0, A1
*
********

Fire_Bullet:
         lea      bullets_data(a5),a0
         move.w   num_b(a5),d0
loop_find_bullet:
         tst.w    (a0)
         beq.s    found_bullet
         lea      b_size(a0),a0
         dbra     d0,loop_find_bullet
rts420:  rts

found_bullet:
         move.l   (a1)+,(a0)+                ; Bullet type, damage
         move.w   d7,(a0)+                   ; X-Coordinate
         move.w   (a1)+,(a0)+                ; Width
         move.w   d6,(a0)+                   ; Y-Coordinate
         move.l   (a1)+,(a0)+                ; Height, image
         move.w   d5,(a0)+                   ; Misc. data
         rts

********************************************* BULLET DESCRIPTIONS
*
* Type, damage, width, height, image
*
********

Bullet_Standard:
         dc.w     2,2,3,10,in_Disruptor_Blast    
Bullet_Triple_Middle:
         dc.w     4,2,4,4,in_Triple_Shot
Bullet_Triple_Left:
         dc.w     6,2,4,4,in_Triple_Shot
Bullet_Triple_Right:
         dc.w     8,2,4,4,in_Triple_Shot
Bullet_Quad_Left:
         dc.w     10,2,5,7,in_Quad_Blast
Bullet_Quad_Right:
         dc.w     12,2,5,7,in_Quad_Blast
Bullet_Plasma:
         dc.w     14,8,5,11,in_Plasma_Blast
Bullet_Arch_Left:
         dc.w     16,16,4,6,in_Blob_Blast
Bullet_Arch_Right:
         dc.w     18,16,4,6,in_Blob_Blast
Bullet_Ultimate1:
         dc.w     20,200,5,10,in_Ultimate_Bullet
Bullet_Ultimate2:
         dc.w     22,200,5,10,in_Ultimate_Bullet
Bullet_Ultimate3:
         dc.w     24,200,5,10,in_Ultimate_Bullet
Bullet_Ultimate4:
         dc.w     26,200,5,10,in_Ultimate_Bullet

********************************************* HANDLE SPECIAL WEAPONS

Special_Weapon_Shoot:
         move.w   #$ffef,$18(a6)
         cmp.w    #2,player_ship(a5)
         beq.s    Special_Weapon_2
         cmp.w    #1,player_ship(a5)
         beq.s    Special_Weapon_1
         rts

********************************************* SPECIAL WEAPON FOR SHIP II

Special_Weapon_1:
         tst.w    special_weapon_delay(a5)
         bne.s    swq
         btst     #0,$1b(a6)
         bne.s    swq
         addq.w   #1,special_weapon_delay(a5)

         lea      bullets_data(a5),a0
         move.w   num_b(a5),d0
         lsl.w    #4,d0
         add.w    d0,a0                      ; A0 -> last bullet

         move.w   #-2,(a0)+
         move.w   #3,(a0)+
         move.w   player_xc(a5),d0
         subq.w   #4,d0
         and.w    #$FFF8,d0
         move.w   d0,(a0)+
         move.w   #32,(a0)+
         move.w   player_yc(a5),d0
         subq.w   #4,d0
         move.w   d0,(a0)+
         move.w   #3,(A0)+
         move.w   #-2,(a0)+
         move.w   #16,(a0)+

swq:     rts

********************************************* SPECIAL WEAPON FOR SHIP III

Special_Weapon_2:
         move.w   special_weapon_delay(a5),d0
         beq.s    swr
         subq.w   #1,special_weapon_delay(a5)
         rts

swr:     btst     #0,$1b(a6)
         bne.s    swq

         move.w   #160,special_weapon_delay(a5)

         lea      enemy_bullets(a5),a0
         moveq    #15,d0
kill_ebullets:
         clr.w    (a0)
         lea      eb_size(a0),a0
         dbra     d0,kill_ebullets

         lea      bullets_data(a5),a0
         move.w   num_b(a5),d0
         subq.w   #8,d0
kill_bullets:
         clr.w    (a0)
         lea      b_size(a0),a0
         dbra     d0,kill_bullets

         moveq    #7,d0
         lea      sw3_angles(pc),a1
         move.w   player_xc(a5),d2
         subq.w   #6,d2
         move.w   player_yc(a5),d1
         subq.w   #5,d2
sw3_deploy:
         move.w   #28,(a0)+
         move.w   #4,(a0)+
         move.w   d2,(a0)+
         addq.w   #2,d2
         move.w   #7,(a0)+
         move.w   d1,(a0)+
         move.w   #7,(a0)+
         move.w   #in_Ball,(a0)+
         move.w   (a1)+,(a0)+
         dbra     d0,sw3_deploy
         rts

sw3_angles:
         dc.b     -5,-2,-4,-3,-3,-5,-1,-5
         dc.b     1,-5,3,-5,4,-3,5,-2
         rts
